%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Matlab file: Moments_SETAR200_Gauss.m 
%% 
%% Coded by: JDG
%%
%% Purpose: Compute EXACT moments, ACF of SETAR(2;0,0) (also called
%%          PCM) with N(0,1) innovations
%% 
%% References:
%%  Li, D. (2010). 
%%    Statistical Inference for Some Threshold Models, PhD. thesis.
%%
%%  Li, D., Ling, S., and Tong, H. (2012).
%%    On moving-average models with feedback. 
%%    Bernoulli, 18, 735--745.
%%
%%  De Gooijer, J.G. (2022). The marginal distribution function of
%%    threshold-type processes with central symmetric innovations,
%%    Statistics, 56.
%% 
%% SETAR(2,0,0) Process = Piecewise Constant Model (PCM) in De Gooijer (2022)
%%    Y_{t} =  mu_1 + varepsilon_{t} if Y_{t-1}<= r,
%%    Y_{t} =  mu_2 + varepsilon_{t} if Y_{t-1}> r,
%% with  varepsilon_{t} ~ G(0,sigma^2), and 
%%       mu_1 = alpha_m;  mu_2 = alpha_p
%%
%% r   = unspecified threshold parameter
%% G   = normcdf  N(0,1)
%%
%% OUTPUT:
%% Parameters SETMA(2;0,0): alpha_m and alpha_p, N(0,1) innovations 
%%   -1.2500    1.2500
%% Statistics: Mean Var  skewness  kurtosis  
%%    0.0000    2.5625   -0.0000    2.2564
%% Theoretical ACVF and ACF of SETAR(2;0,0) at lag k: 
%%    1.0000    1.6890    0.6591
%%    2.0000    1.3321    0.5198
%%    3.0000    1.0506    0.4100
%%    4.0000    0.8286    0.3234
%%    5.0000    0.6535    0.2550
%%    6.0000    0.5154    0.2011
%%    7.0000    0.4065    0.1586
%%    8.0000    0.3206    0.1251
%%    9.0000    0.2529    0.0987
%%   10.0000    0.1994    0.0778
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; clc;
sigma = 1;

alpha_m = -1.25;   % Example 3.2 (and Figure 1) in De Gooijer (2022)
alpha_p = -alpha_m;
par     = [alpha_m, alpha_p];

Ee_3  = 0;
Ee_4  = 3;  % 4th order moment N(0,1)
r     = 0;
lag   = 10; % maximum number of lags

%% omega = weight. Notation changed, replaces delta_{0} in Li et al. paper
omega  = normcdf(r-alpha_p)/(1-normcdf(r-alpha_m)+normcdf(r-alpha_p));  
alpha  = normcdf(r-alpha_p);
beta   = normcdf(r-alpha_m)-normcdf(r-alpha_p); 

% Mean and variance; see Li PhD thesis, p. 12
nu_1_S200 = alpha_p +(alpha_m - alpha_p) * omega;    
Var_S200  = sigma^(2) + (alpha_m - alpha_p)^(2) * omega*(1-omega);
sd_S200   = sqrt(Var_S200); 

%% ACVF, Cov(y_{n},y_{n-1}), and ACF(1)

for k=1:1   % lag 1
  lambda2 = -(alpha_m - alpha_p) * ( omega*normpdf(r-alpha_m) + ...
             (1-omega)*normpdf(r-alpha_p) );   % minus sign!
            
  %% Term2 is 2nd term of Cov in Li PhD, p. 13 and in Li et al. (2012) rho_{k}          
  term2   = (alpha_m - alpha_p)^(2) * omega * (1-omega)*beta^(k);  
  Cov(k)  = lambda2 + term2;
            
  ACVF_S200(k) = Cov(k) + nu_1_S200^(2);           
  ACF_S200(k)  = Cov(k)/Var_S200; 
end

lambda_k1   = ( omega*normpdf(r-alpha_m) + (1-omega)*normpdf(r-alpha_p) );
lambda_k2   = normpdf(r-alpha_m); 

Cov_k1 = (alpha_m - alpha_p) * (-lambda_k1) +term2;

rho_lag1  = ( -2*1.25 * (-lambda_k2) + beta * (alpha_p)^(2) )/Var_S200;  
acvf_lag1 = ( -2*1.25 * (-lambda_k2) + beta * (alpha_p)^(2) );           

term3 = -(alpha_m-alpha_p) * ( omega *normpdf(r-alpha_m) + ...
                             (1-omega)*normpdf(r-alpha_p) ); 

term4 = ( alpha_m^(2)*omega * normcdf(r-alpha_m) + ...
           alpha_p^(2)*(1-omega)*(1-normcdf(r-alpha_p) ) + ...
        2*alpha_m * alpha_p * omega * (1-omega)*(1-beta) );  
        
term5 = -term3*( normcdf(r-alpha_p) - normcdf(r-alpha_m) );  
term6 = -term4*( normcdf(r-alpha_p) - normcdf(r-alpha_m) );  
ACVF2 = term5 + term6;  

term7 = -term5*( normcdf(r-alpha_p) - normcdf(r-alpha_m) );  
term8 = -term6*( normcdf(r-alpha_p) - normcdf(r-alpha_m) );  
ACVF3 = term7 + term8;  

tt(1)        = term3;
rr(1)        = term4;
acvf_S200(1) = term3 + rr(1);  % ACVF at lag 1
acf_S200(1)  = ACF_S200(1);    % ACF  at lag 1

ttt(1)     = term3;
rrr(1)     = term4;
acvf_S2(1) = term3+term4;
acf_S2(1)  = acvf_S2(1)/Var_S200;

for k=2:10
 ttt(k)     = (-1)^(k-1)*term3 *( normcdf(r-alpha_p) - normcdf(r-alpha_m) )^(k-1);
 rrr(k)     = (-1)^(k-1)*term4 *( normcdf(r-alpha_p) - normcdf(r-alpha_m) )^(k-1);
 acvf_S2(k) = ttt(k) + rrr(k); 
 acf_S2(k)  = acvf_S2(k)/Var_S200;
end

%% Skewness and kurtosis 
skew_S200 = ( Ee_3 + (alpha_m - alpha_p)^(3) * (omega   - ...
              3*omega^(2) + 2*omega^(3)) )/sd_S200^(3); 
              
kurt_S200 = ( Ee_4 + 6*sigma^(2)*(alpha_m - alpha_p)^(2) * omega * (1-omega) + ...
 (alpha_m-alpha_p)^(4)*(omega-4*omega^(2)+6*omega^(3)-3*omega^(4)) )/sd_S200^(4);

%%%%%%%%%%%%%%  OUTPUT   %%%%%%%%%%%%%%%%%%%%%
disp('Parameters SETMA(2;0,0): alpha_m and alpha_p, N(0,1) innovations ')
disp(par)

disp('Statistics: Mean Var  skewness  kurtosis  ')
moments = [nu_1_S200, Var_S200, skew_S200, kurt_S200];
disp(moments)

disp('Theoretical ACVF and ACF of SETAR(2;0,0) at lag k: ')
for k=1:lag
  ACF_S2_tot = [k, acvf_S2(k), acf_S2(k) ];
  disp(ACF_S2_tot)
end

%%%%%%%%%%%%%%%%%%% END MAIN  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%