%%  File: Skew_Kurt_CasMA_Normal.m
%%
%%  Coded by: JDG
%%
%%  pdf of Contemporaneous asMA(q,q) process (CasMA), skewness, 
%%  kurtosis, and ACF at lags 1 and 2:
%%  
%%  Reference:
%%  Guay, A. and Scaillet, O. (2003).
%%  "Indirect inference, nuisance parameter, and 
%%   threshold moving average models", JBES, 2003, 21, 122-132.
%%
%%    Y_{t} =  mu + D^{+}(L)varepsilon^{+}_{t}
%%                + D^{-}(L)varepsilon^{-}_{t}
%%  with varepsilon^{+}_{t} = varepsilon I(varepsilon >r)       
%%             varepsilon_{t} ~ N(0,1) 
%%
%%  D^{+}(L) = d0^{+} + d1^{+}|L + ... + dq^{+}L^{q}
%%  D^{-}(L) = d0^{-} + d1^{-}|L + ... + dq^{-}L^{q}
%%
%%  Define:
%%  U_{t} = Y_{t} - E[Y_{t}|varepsilon_{t-1}] 
%%        = d0^{+}eta^{+}_{t} + d0^{-}eta^{-}_{t} 
%%  eta^{+,-} = varepsilon^{+,-} - E[varepsilon^{+,-}_{t}]
%%  varepsilon^{+}_{t} = varepsilon_{t} I(varepsilon > gamma)
%%
%% Model: Y_{t} = d0_p * varepsilon^{+}_{t} + d0_m * varepsilon^{-}_{t}
%%                 with d0_p = -d0_m = 1, varepsilon ~ N(0,1)
%%
%% OUTPUT: PART 1
%% Simulation results:
%% Parameters for CasMA(1,1) d0_m, d0_p
%%     1
%% Central moments:   nu3    nu4
%%                  3.4336   20.1698
%%   SD,    Skewness, Kurtosis
%%  2.2832   -0.9953    3.8692
%%  1.8494   -0.9953    3.8692
%%  1.4612   -0.9953    3.8692
%%  1.1188   -0.9953    3.8692
%%  0.8219   -0.9953    3.8692
%%  0.5708   -0.9953    3.8692
%%  0.3653   -0.9953    3.8692
%%  0.2055   -0.9953    3.8692
%%  0.0913   -0.9953    3.8692
%%  0.0228   -0.9953    3.8692
%%       0       NaN       NaN
%%  0.0228    0.9953    3.8692
%%  0.0913    0.9953    3.8692
%%  0.2055    0.9953    3.8692
%%  0.3653    0.9953    3.8692
%%  0.5708    0.9953    3.8692
%%  0.8219    0.9953    3.8692
%%  1.1188    0.9953    3.8692
%%  1.4612    0.9953    3.8692
%%  1.8494    0.9953    3.8692
%%  2.2832    0.9953    3.8692
%%
%% PART 2:
%% Parameters: +, - 
%%    1.0000    0.8000    0.4000   -1.0000   -0.8000   -0.4000
%%    acf(1),   acf(2)
%%    0.6748    0.2410
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; clc;

%% Special case gamma = r = 0 threshold parameter
r    = 0;  % gamma in JBES paper
mu   = 0;  % constant term

Par  = 21;
kurt = zeros(Par,1);
skew = zeros(Par,1);
sd   = zeros(Par,1);

%%%%%%%%%%%%  Part 1 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i=1:Par
  par   = -1+(i-1)*0.1;
  d0_p  = par;
  d0_m  = -d0_p;   % +0.2;

  % Central moments
  mu_3 = ( d0_p^(3) - d0_m^(3) ) * (pi+2) + ...
            3 * ( d0_p^(2) * d0_m - d0_p * d0_m^(2) ) * (pi-2);
  mu_2 = ( d0_p^(2) + d0_m^(2) ) * (pi-1) + 2*d0_p*d0_m;
  %%  term4 = sd/(2*pi);  % see below

  % Conditional kurtosis of Y_{t}:
  mu_4 = ( d0_p^(4)+d0_m^(4) ) * (6*pi^(2) - 10*pi -3) + ...
            4 * ( d0_p^(3)*d0_m + d0_p * d0_m^(3) ) *(pi+3) + ...
            6 * d0_p^(2) *d0_m^(2) * (2*pi - 3);

  %% Conditional skewness, kurtosis and standard deviation:        
  skew(i) = mu_3/mu_2^(1.5);         
  kurt(i) = mu_4/mu_2^(2);
  sd(i)   = mu_2;
  
end % end i=1:Par

disp('Simulation results:')
disp('Parameters for CasMA(1,1) d0_m, d0_p')
disp(par)
mu_tot = [mu_3, mu_4];
disp('Central moments:   nu3    nu4' )
disp(mu_tot)
disp(' ')

Moments = [sd, skew, kurt];
disp('SD, Skewness, Kurtosis')
disp(Moments)
disp(' ')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% General case gamma = r (threshold parameter) NOT EQUAL 0
%% c(i,j) = E[(eta^{+})^(i) * eta^{-})^(j)] i,j>=1
%%
%% Skewness and kurtosis as a function of "r"

Umax   = 41;
skew_r = zeros(1,Umax);
kurt_r = zeros(1,Umax);

for i=1:Umax
  r       = -2+(i-1)*0.1;
  n_pdf   = normpdf(r,0,1);
  n_cdf_p = normcdf(r);
  n_cdf_m = normcdf(-r);

%% Moments of centered processes:
%% eta^{+}_{t}=varespilon^{+}_{t}-E[varepsilon_{t}^{+}]
%% eta^{-}_{t}=varespilon^{-}_{t}-E[varepsilon_{t}^{-}]
%
% Note: c(3,1) = c(2,0) = E[(eta^{+}_{t})^(2)] in Appendix of JBES paper
%       c(4,1) = c(3,0) = E[(eta^{+}_{t})^(3)] 
%       c(5,1) = c(4,0) = E[(eta^{+}_{t})^(4)]
%       c(1,3) = c(0,2) = E[(eta^{-}_{t})^(2)]
%       etc.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  c(3,1) = r * n_pdf - n_pdf^(2) + n_cdf_p;
  c(4,1) = (r^(2)+ 2)*n_pdf - 3*r*n_pdf^(2) - 3*n_pdf*n_cdf_m + 2*n_pdf^(3);
  c(5,1) = (r^(3)+ 3*r)*n_pdf - ( 4*(r^(2)+2) - 6*n_cdf_m )*n_pdf^(2) + ...
                      6*r*n_pdf^(3) - 3*n_pdf^(4) + 3*n_cdf_m;

  c(1,3) = -r * n_pdf - n_pdf^(2) + n_cdf_p;
  c(1,4) = -(r^(2)+ 2)*n_pdf - 3*r*n_pdf^(2) + 3*n_pdf*n_cdf_m - 2*n_pdf^(3);
  c(1,5) = -(r^(3)+ 3*r)*n_pdf - ( 4*(r^(2)+2) - 6*n_cdf_m )*n_pdf^(2) - ...
                       6*r*n_pdf^(3) - 3*n_pdf^(4) + 3*n_cdf_m;
           
  c(2,2) = n_pdf^(2);
  c(3,2) = -2*n_pdf^(3) + r*n_pdf^(2) + n_pdf*n_cdf_m;
  c(2,3) =  2*n_pdf^(3) + r*n_pdf^(2) - n_pdf*n_cdf_m;
  c(3,3) =  n_pdf^(2)*( n_cdf_p + n_cdf_m) - 3*n_pdf^(4);

  c(4,2) = 3*n_pdf^(4) + n_pdf^(2) * (r^(2) + 2 -3*(r*n_pdf+n_cdf_m) ) ;
  c(2,4) = 3*n_pdf^(4) + n_pdf^(2) * (r^(2) + 2 +3*(r*n_pdf-n_cdf_p) ) ;

  term3 = d0_p^(0)*d0_m^(3)*c(1,4) + 3*d0_p*d0_m^(2)*c(2,3) + ...
          3*d0_p^(2)*d0_m*c(3,2)   + ...
          d0_p^(3)*d0_m^(0)*c(4,1); % numerator of skewness
        
  term4 = d0_p^(0)*d0_m^(2)*c(1,3) + 2*d0_p*d0_m*c(2,2) + ...
          d0_p^(2)*d0_m^(0)*c(3,1); % std for denominator of skewness + kurtosis
        
  term5 = d0_p^(0)*d0_m^(4)*c(1,5)   + 4*d0_p^(1)*d0_m^(3)*c(2,4) + ...
          6*d0_p^(2)*d0_m^(2)*c(3,3) + 4*d0_p^(3)*d0_m^(1)*c(4,2) + ...
          d0_p^(4)*d0_m^(0)*c(5,1); % numerator of kurtosis
              
  %% Conditional skewness and kurtosis, general value of "r"       
  skew_r(i) = term3/(term4)^(3/2); 
  kurt_r(i) = term5/(term4)^(2);
  index(i)  = r;
end  % for i=1:Umax

figure(1)
plot(index,skew_r)

figure(2)
plot(index,kurt_r)

figure(3)
plot(index(20:end),kurt_r(20:end))

skew_all = [index', skew_r'];
kurt_all = [index', kurt_r'];

%%%%%%%%%%  ACVF at lags 1 and 2  %%%%%%%%%%%%%%%%%%%%%%%%%%%
%% CasMA with q = 2 lags (q = ell in JBES paper), r = 0 
%%  D^{+}(L) = d0^{+} + d1^{+}|L + ... + dq^{+}L^{q}
%%  D^{-}(L) = d0^{-} + d1^{-}|L + ... + dq^{-}L^{q}
%%     varepsilon_{t} ~ N(0,1)
%%
%% Proposition 2:
%% acvf(u) = sum^{q-u}_{i=0}[ (
%%           d_{i+u}^{+} d_{i}^{+}E[(eta^{+}_{t-i})^2] +
%%           d_{i+u}^{-} d_{i}^{-}E[(eta^{-}_{t-i})^2] ) +
%% ( d_{i+u}^{+}d_{i}^{-} + d_{i+u}^{-}d_{i}^{+} ) * 
%%                           E[(eta^{+}_{t-i}eta^{-}_{t-i}] ] 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%  Part 2 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all; clc;

d0_p = 1; 
d1_p = 0.8;
d2_p = 0.4;

d0_m = -d0_p;
d1_m = -0.8;
d2_m = -0.4;

par = [d0_p, d1_p, d2_p, d0_m, d1_m, d2_m];

r       = 0;
n_pdf   = normpdf(r,0,1);
n_cdf_p = normcdf(r);
n_cdf_m = normcdf(-r);

c(3,1) =  r * n_pdf - n_pdf^(2) + n_cdf_p;
c(1,3) = -r * n_pdf - n_pdf^(2) + n_cdf_p;
c(2,2) = n_pdf^(2);

%% acvf(u) for u<=q; acvf(u) = 0 for u>q  (here q=2)
acvf_0  =  d0_p*d0_p*c(3,1) + d0_m*d0_m*c(1,3) + ...
           d1_p*d1_p*c(3,1) + d1_m*d1_m*c(1,3) + ...
           d2_p*d2_p*c(3,1) + d2_m*d2_m*c(1,3) + ...          
           ( d0_p*d0_m + d0_m*d0_p )*c(2,2) + ...
           ( d1_p*d1_m + d1_m*d1_p )*c(2,2) + ...
           ( d2_p*d1_m + d2_m*d1_p )*c(2,2);

acvf(1) =  d1_p*d0_p*c(3,1) + d1_m*d0_m*c(1,3) + ...
           d2_p*d1_p*c(3,1) + d2_m*d1_m*c(1,3) + ...
           ( d1_p*d0_m + d1_m*d0_p )*c(2,2) + ...
           ( d2_p*d1_m + d2_m*d1_p )*c(2,2);
                      
acvf(2) = d2_p*d0_p * c(3,1) + d2_m*d0_m*c(1,3) + ...
           ( d2_p*d0_m + d2_m*d0_p )*c(2,2);
           
acf(1) = acvf(1)/acvf_0;
acf(2) = acvf(2)/acvf_0;

ACF = [acf(1),acf(2)];
disp('Parameters: +, - ')
disp(par)
disp(' ')
disp('acf(1)   acf(2)')
disp(ACF)
disp(' ')

%%%%%%%%%%%%%%%%%%%%% END CODE  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


