% Matlab file: pdf_ARCH_Gaussian_discretization.m
%
% NOTE: ML code is computationally demanding
% 
% Purpose: Compute pdf of discrete AR(1) process with ARCH(1) errors.
% Y_{t} = alpha Y_{t-1} + (beta + lambda Y^{2}_{1-1})^{1/2}\nu_{t} (t>=1)
% where nu_{t} are i.i.d. real-valued rvs independent from Y_{0}.
%
% input file: stationary.m  (see below)
%
% Reference:
%  Herve and Ledoux (2020). State-discretization of 
%    V-geometrically ergodic Markov chains and convergence to
%    stationarity, Methodology and Computing in Applied Probability,
%    22: 905--925.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; clc;

% Set the parameters: alpha >> lambda beta=1
beta   = 1;

alpha  = 0.7;
lambda = 0.2; 
% Transition kernel in Eq. (34), p. 916
% p(x,y) = (beta+lambda x^2)^{-1/2} \nu[ (y-alpha x)/(beta + lambda x^2)^{1/2} ]
% with nu(.) = pdf of N(0,1)
%
p=@(x,y) (beta+ ... 
   lambda*x.^2).^(-0.5)*(2*pi)^(-0.5).*exp(-(y-alpha*x).^2./(2*(beta+lambda*x.^2)));

% Define the density of the errors
% Gauss
sigma = 1;
mu    = 0;
nu    = @(x) pdf('normal',x,mu,sigma);
% fplot(nu,[-4,4]);

deltak = 0.02;
% Values of "kmin" and "kmax"
kmin = -15;
kmax = 15;
qmax = (kmax-kmin)/deltak;

% grid points
xk = zeros(qmax+1,1);
for i=1:qmax+1
    xk(i) = kmin+(i-1)*deltak;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Construct the matrix B_k
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialization
Bk = zeros(qmax+1,qmax+1);
% All rows and columns except the last one
for i=1:qmax 
    nuik = @(y) min(p(xk(i),y), p(xk(i+1),y) ); %  Define nu_{i,k} 
    for j =1:qmax
        Bk(i,j) = quad(nuik,xk(j),xk(j+1));
    end
end

%% Choose the column j_{0}\in {1,...,qkmax} and set column j_{0} of B_{k} 
j0 = floor(qmax/2);
for i=1:qmax 
    Bk(i,j0+1) = Bk(i,j0+1) + 1 - quad(nuik,kmin,kmax); 
end

Bk(qmax+1,j0+1) = 1;

% Compute the invariant probabilities B_{k}
pik = stationary(Bk);

% Define the pdf pk
% set-up a grid
x  = kmin:(kmax-kmin)/100:kmax;
pk = [];
for y = kmin:(kmax-kmin)/100:kmax  
    val =0;
    for i=1:qmax 
        nuik = @(y) min(p(xk(i),y), p(xk(i+1),y));  % Define nu_{i,k}
        val  = val + pik(i)*nuik(y);
    end
    pk = [pk,val];
end

%% Figure 1 (Gauss), 2nd figure from top. Case: rho =0.7
dessin = plot(x,pk,'r',x,nu(x),'b');
hold on 
legend(dessin,'estimated invariant pdf','innovation pdf');
hold off;

%%%%%%%%%%%%%%%%%%%%  END MAIN %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%% Input file %%%%%%%%%%%%%%%%%%%%%%%%

function PI = stationary(P)
% Computes the stationary distribution of an ergodic Markov chain
% caracterized by its transition probabilities matrix A such that
% A = Pr(x_{k} | x_{k - 1}) , sum(A , 2) = 1.
%
% PI = stationary(P);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[M, N] = size(P);
if (M ~= N) 
    error('Matrix A not square')   
end

[V , D ] = eigs(P');
[foo,tp] = sort(diag(D));
PI       = (V(: , tp(end))/sum(V(: , tp(end))))';
