% Matlab file: pdf_Exponential_discretization.m
% 
% Exponential innovation distribution: Exp(nu)
%
% input file: stationary.m (see below)
%
% Reference:
%  Herve, L. and Ledoux, J. (2020). State-discretization of 
%    V-geometrically ergodic Markov chains and convergence to
%    stationarity, 
%    Methodology and Computing in Applied Probability,
%    22: 905--925.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; clc;

%% WARNING:
%  Set the value of "rho" (the default is:): 
   rho1 = 0.7;

% If fixed at order 3 the Markov chain has the matrix "P^3" and "rho" must be
% replaced by "rho^3"
  rho = (rho1)^3;

% Define the error processes:
% Exp(1) distribution:
% nu = @(x) heaviside(x).*exp(-x);         % nu_{1} or h_{0}
% fplot(nu,[0,10]) integral(nu,0,10) = 1 

% Exp(2) distribution:
% nu = @(x) heaviside(x).*exp(-x).*(1-exp(x-x/rho1))/(1-rho1); % nu_{2} or h_{1}
% fplot(nu,[0,12]) with rho=0.9: integral(nu,0,12)=1
%                       rho=0.7: integral(nu,0,12)=1
%                       rho=0.5: integral(nu,0,11)=1

% Exp(3) distribution:
% nu = @(x) heaviside(x).*(exp(-x)-exp(-x/rho)+ rho*(-exp(-x)+exp(-x/rho^2))/(1+rho)) / (1-rho)^2; % \nu_3 ou h_2
  nu = @(x) (x>=0).*(exp(-x)-exp(-x/rho1)+ rho1*(-exp(-x)+exp(-x/rho1^2))/(1+rho1)) / (1-rho1)^2; % \nu_3 ou h_2

% The support is based on the integral with mu_{3}
%========================================================================
% The Figs. 2-4 in the paper by Herve and Ledoux are computed with 
% fplot(nu,[0,14]) with rho1=0.9: integral(nu,0,14)=1
%                       rho1=0.7: integral(nu,0,12)=1
%                       rho1=0.5: integral(nu,0,11)=1
% for nu_rho=0.5 the support = [0,11] with [0,11/(1-(0.5)^3)] -> [0,13] 
%  ,, nu_rho=0.6    ,,       = [0,12] with [0,12/(1-(0.6)^3)] -> [0,16] 
%  ,, nu_rho=0.7    ,,       = [0,12] with [0,12/(1-(0.7)^3)] -> [0,19]  
%  ,, nu_rho=0.8    ,,       = [0,13] with [0,13/(1-(0.8)^3)] -> [0,27]
%  ,, nu_rho=0.9    ,,       = [0,14] with [0,14/(1-(0.9)^3)] -> [0,52] 
%=================================================================================================

% The order of the innovations:
m = 2;

% Define the function "V" (the value of "m" is globally defined)
V = @(x) 1+abs(x)^m;

% deltak = 1/V(k);
deltak   = 0.02;

% Values of "kmin" and "kmax"
kmin = 0;
kmax = 20; % 52 (blue), 48 (red), 56 (yellow)

qmax = (kmax-kmin)/deltak;
% Generate the grid points
xk = zeros(qmax+1);
for i=1:qmax+1
   xk(i) = kmin+(i-1)*deltak;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Construct the matrix B_{k}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialization: 
Bk = zeros(qmax+1,qmax+1); % All rows and columns except the last ones

% All rows and columns except the last ones
for i=1:qmax 
    nuik = @(y) min(nu(y-rho*xk(i)), nu(y-rho*xk(i+1))); % Define nu_{i,k} 
    for j=1:qmax
        Bk(i,j) = quad(nuik,xk(j),xk(j+1));
    end
end

% Choose the column j_{0}\in {1,...,qkmax} and set column j_{0} of B_{k} 
j0 = floor(qmax/2);
for i=1:qmax 
    Bk(i,j0+1) = Bk(i,j0+1) + 1 - integral(nuik,kmin,kmax); 
end
Bk(qmax+1,j0+1) = 1;

% Compute the invariant probabilities "$B_{k}$"
pik = stationary(Bk);

% Define the pdf "p_{k}"
% set-up a grid
x  = kmin:(kmax-kmin)/100:kmax;
pk = [];
for y = kmin:(kmax-kmin)/100:kmax  
    % Compute pk(y)
    val = 0;
    for i=1:qmax 
        nuik = @(y) min(nu(y-rho*xk(i)), nu(y-rho*xk(i+1))); % Define nu_{i,k}
        val  = val + pik(i)*nuik(y);
    end
    pk = [pk,val];
end

% Save the densities 
% save 'pdf_rho09_kmax50_kmin0_deltak001.mat'  * pk

%% Figure 2 (Expo), second figure from top. Case: rho =0.7
dessin = plot(x,pk,'r',x,nu(x),'b');
hold on 
legend(dessin,'estimated invariant pdf','innovation pdf');
hold off;

%%%%%%%%%%%%%%%%%%%%  END MAIN %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%% Input file %%%%%%%%%%%%%%%%%%%%%%%%
function PI = stationary(P)

% Compute the stationary distribution of an ergodic Markov chain
% characterized by its transition probabilities matrix A such that
% A = Pr(x_{k} | x_{k - 1}), sum(A, 2) = 1.
%
% PI = stationary(P);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[M, N] = size(P);

if (M ~= N) 
    error('Matrix A not square')   
end

[V, D ]   = eigs(P'); 
[foo, tp] = sort(diag(D));
PI        = (V(:,tp(end))/sum(V(:,tp(end))))';

